@@ -24,6 +24,7 @@ Three convenience functions are also provided for common flag constraints:
     flags.mark_flag_as_required
     flags.mark_flags_as_required
     flags.mark_flags_as_mutual_exclusive
+    flags.mark_bool_flags_as_mutual_exclusive
 
 See their docstring in this module for a usage manual.
 
@@ -39,157 +40,7 @@ import warnings
 
 from absl.flags import _exceptions
 from absl.flags import _flagvalues
-
-
-class Validator(object):
-  """Base class for flags validators.
-
-  Users should NOT overload these classes, and use flags.Register...
-  methods instead.
-  """
-
-  # Used to assign each validator an unique insertion_index
-  validators_count = 0
-
-  def __init__(self, checker, message):
-    """Constructor to create all validators.
-
-    Args:
-      checker: function to verify the constraint.
-          Input of this method varies, see SingleFlagValidator and
-          multi_flags_validator for a detailed description.
-      message: str, error message to be shown to the user.
-    """
-    self.checker = checker
-    self.message = message
-    Validator.validators_count += 1
-    # Used to assert validators in the order they were registered.
-    self.insertion_index = Validator.validators_count
-
-  def verify(self, flag_values):
-    """Verifies that constraint is satisfied.
-
-    flags library calls this method to verify Validator's constraint.
-
-    Args:
-      flag_values: flags.FlagValues, the FlagValues instance to get flags from.
-    Raises:
-      Error: Raised if constraint is not satisfied.
-    """
-    param = self._get_input_to_checker_function(flag_values)
-    if not self.checker(param):
-      raise _exceptions.ValidationError(self.message)
-
-  def get_flags_names(self):
-    """Returns the names of the flags checked by this validator.
-
-    Returns:
-      [string], names of the flags.
-    """
-    raise NotImplementedError('This method should be overloaded')
-
-  def print_flags_with_values(self, flag_values):
-    raise NotImplementedError('This method should be overloaded')
-
-  def _get_input_to_checker_function(self, flag_values):
-    """Given flag values, returns the input to be given to checker.
-
-    Args:
-      flag_values: flags.FlagValues, containing all flags.
-    Returns:
-      The input to be given to checker. The return type depends on the specific
-      validator.
-    """
-    raise NotImplementedError('This method should be overloaded')
-
-
-class SingleFlagValidator(Validator):
-  """Validator behind register_validator() method.
-
-  Validates that a single flag passes its checker function. The checker function
-  takes the flag value and returns True (if value looks fine) or, if flag value
-  is not valid, either returns False or raises an Exception.
-  """
-
-  def __init__(self, flag_name, checker, message):
-    """Constructor.
-
-    Args:
-      flag_name: string, name of the flag.
-      checker: function to verify the validator.
-          input  - value of the corresponding flag (string, boolean, etc).
-          output - bool, True if validator constraint is satisfied.
-              If constraint is not satisfied, it should either return False or
-              raise flags.ValidationError(desired_error_message).
-      message: str, error message to be shown to the user if validator's
-          condition is not satisfied.
-    """
-    super(SingleFlagValidator, self).__init__(checker, message)
-    self.flag_name = flag_name
-
-  def get_flags_names(self):
-    return [self.flag_name]
-
-  def print_flags_with_values(self, flag_values):
-    return 'flag --%s=%s' % (self.flag_name, flag_values[self.flag_name].value)
-
-  def _get_input_to_checker_function(self, flag_values):
-    """Given flag values, returns the input to be given to checker.
-
-    Args:
-      flag_values: flags.FlagValues, the FlagValues instance to get flags from.
-    Returns:
-      object, the input to be given to checker.
-    """
-    return flag_values[self.flag_name].value
-
-
-class MultiFlagsValidator(Validator):
-  """Validator behind register_multi_flags_validator method.
-
-  Validates that flag values pass their common checker function. The checker
-  function takes flag values and returns True (if values look fine) or,
-  if values are not valid, either returns False or raises an Exception.
-  """
-
-  def __init__(self, flag_names, checker, message):
-    """Constructor.
-
-    Args:
-      flag_names: [str], containing names of the flags used by checker.
-      checker: function to verify the validator.
-          input  - dict, with keys() being flag_names, and value for each
-              key being the value of the corresponding flag (string, boolean,
-              etc).
-          output - bool, True if validator constraint is satisfied.
-              If constraint is not satisfied, it should either return False or
-              raise flags.ValidationError(desired_error_message).
-      message: str, error message to be shown to the user if validator's
-          condition is not satisfied
-    """
-    super(MultiFlagsValidator, self).__init__(checker, message)
-    self.flag_names = flag_names
-
-  def _get_input_to_checker_function(self, flag_values):
-    """Given flag values, returns the input to be given to checker.
-
-    Args:
-      flag_values: flags.FlagValues, the FlagValues instance to get flags from.
-    Returns:
-      dict, with keys() being self.lag_names, and value for each key
-      being the value of the corresponding flag (string, boolean, etc).
-    """
-    return dict([key, flag_values[key].value] for key in self.flag_names)
-
-  def print_flags_with_values(self, flag_values):
-    prefix = 'flags '
-    flags_with_values = []
-    for key in self.flag_names:
-      flags_with_values.append('%s=%s' % (key, flag_values[key].value))
-    return prefix + ', '.join(flags_with_values)
-
-  def get_flags_names(self):
-    return self.flag_names
+from absl.flags import _validators_classes
 
 
 def register_validator(flag_name,
@@ -218,7 +69,7 @@ def register_validator(flag_name,
     AttributeError: Raised when flag_name is not registered as a valid flag
         name.
   """
-  v = SingleFlagValidator(flag_name, checker, message)
+  v = _validators_classes.SingleFlagValidator(flag_name, checker, message)
   _add_validator(flag_values, v)
 
 
@@ -282,7 +133,7 @@ def register_multi_flags_validator(flag_names,
   Raises:
     AttributeError: Raised when a flag is not registered as a valid flag name.
   """
-  v = MultiFlagsValidator(
+  v = _validators_classes.MultiFlagsValidator(
       flag_names, multi_flags_checker, message)
   _add_validator(flag_values, v)
 
@@ -333,16 +184,13 @@ def mark_flag_as_required(flag_name, flag_values=_flagvalues.FLAGS):
   Important note: validator will pass for any non-None value, such as False,
   0 (zero), '' (empty string) and so on.
 
-  It is recommended to call this method like this:
+  If your module might be imported by others, and you only wish to make the flag
+  required when the module is directly executed, call this method like this:
 
     if __name__ == '__main__':
       flags.mark_flag_as_required('your_flag_name')
       app.run()
 
-  Because validation happens at app.run() we want to ensure required-ness
-  is enforced at that time. You generally do not want to force users who import
-  your code to have additional required flags for their own binaries or tests.
-
   Args:
     flag_name: str, name of the flag
     flag_values: flags.FlagValues, optional FlagValues instance where the flag
@@ -355,7 +203,8 @@ def mark_flag_as_required(flag_name, flag_values=_flagvalues.FLAGS):
     warnings.warn(
         'Flag --%s has a non-None default value; therefore, '
         'mark_flag_as_required will pass even if flag is not specified in the '
-        'command line!' % flag_name)
+        'command line!' % flag_name,
+        stacklevel=2)
   register_validator(
       flag_name,
       lambda value: value is not None,
@@ -366,7 +215,8 @@ def mark_flag_as_required(flag_name, flag_values=_flagvalues.FLAGS):
 def mark_flags_as_required(flag_names, flag_values=_flagvalues.FLAGS):
   """Ensures that flags are not None during program execution.
 
-  Recommended usage:
+  If your module might be imported by others, and you only wish to make the flag
+  required when the module is directly executed, call this method like this:
 
     if __name__ == '__main__':
       flags.mark_flags_as_required(['flag1', 'flag2', 'flag3'])
@@ -406,7 +256,8 @@ def mark_flags_as_mutual_exclusive(flag_names, required=False,
       warnings.warn(
           'Flag --{} has a non-None default value. That does not make sense '
           'with mark_flags_as_mutual_exclusive, which checks whether the '
-          'listed flags have a value other than None.'.format(flag_name))
+          'listed flags have a value other than None.'.format(flag_name),
+          stacklevel=2)
 
   def validate_mutual_exclusion(flags_dict):
     flag_count = sum(1 for val in flags_dict.values() if val is not None)
@@ -420,6 +271,35 @@ def mark_flags_as_mutual_exclusive(flag_names, required=False,
       flag_names, validate_mutual_exclusion, flag_values=flag_values)
 
 
+def mark_bool_flags_as_mutual_exclusive(flag_names, required=False,
+                                        flag_values=_flagvalues.FLAGS):
+  """Ensures that only one flag among flag_names is True.
+
+  Args:
+    flag_names: [str], names of the flags.
+    required: bool. If true, exactly one flag must be True. Otherwise, at most
+        one flag can be True, and it is valid for all flags to be False.
+    flag_values: flags.FlagValues, optional FlagValues instance where the flags
+        are defined.
+  """
+  for flag_name in flag_names:
+    if not flag_values[flag_name].boolean:
+      raise _exceptions.ValidationError(
+          'Flag --{} is not Boolean, which is required for flags used in '
+          'mark_bool_flags_as_mutual_exclusive.'.format(flag_name))
+
+  def validate_boolean_mutual_exclusion(flags_dict):
+    flag_count = sum(bool(val) for val in flags_dict.values())
+    if flag_count == 1 or (not required and flag_count == 0):
+      return True
+    raise _exceptions.ValidationError(
+        '{} one of ({}) must be True.'.format(
+            'Exactly' if required else 'At most', ', '.join(flag_names)))
+
+  register_multi_flags_validator(
+      flag_names, validate_boolean_mutual_exclusion, flag_values=flag_values)
+
+
 def _add_validator(fv, validator_instance):
   """Register new flags validator to be checked.
 
