@@ -71,10 +71,8 @@ program.
 The differences in behavior are historical and unfortunate.
 """
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
+import collections
+from collections import abc
 import getpass
 import io
 import itertools
@@ -83,19 +81,20 @@ import os
 import socket
 import struct
 import sys
+import threading
 import time
 import timeit
 import traceback
+import types
 import warnings
 
 from absl import flags
 from absl.logging import converter
-import six
 
-if six.PY2:
-  import thread as _thread_lib  # For .get_ident().
-else:
-  import threading as _thread_lib  # For .get_ident().
+try:
+  from typing import NoReturn
+except ImportError:
+  pass
 
 
 FLAGS = flags.FLAGS
@@ -174,7 +173,10 @@ class _VerbosityFlag(flags.Flag):
     self._update_logging_levels()
 
   def _update_logging_levels(self):
-    """Updates absl logging levels to the current verbosity."""
+    """Updates absl logging levels to the current verbosity.
+
+    Visibility: module-private
+    """
     if not _absl_logger:
       return
 
@@ -194,6 +196,64 @@ class _VerbosityFlag(flags.Flag):
       _absl_logger.setLevel(standard_verbosity)
 
 
+class _LoggerLevelsFlag(flags.Flag):
+  """Flag class for --logger_levels."""
+
+  def __init__(self, *args, **kwargs):
+    super(_LoggerLevelsFlag, self).__init__(
+        _LoggerLevelsParser(),
+        _LoggerLevelsSerializer(),
+        *args, **kwargs)
+
+  @property
+  def value(self):
+    # For lack of an immutable type, be defensive and return a copy.
+    # Modifications to the dict aren't supported and won't have any affect.
+    # While Py3 could use MappingProxyType, that isn't deepcopy friendly, so
+    # just return a copy.
+    return self._value.copy()
+
+  @value.setter
+  def value(self, v):
+    self._value = {} if v is None else v
+    self._update_logger_levels()
+
+  def _update_logger_levels(self):
+    # Visibility: module-private.
+    # This is called by absl.app.run() during initialization.
+    for name, level in self._value.items():
+      logging.getLogger(name).setLevel(level)
+
+
+class _LoggerLevelsParser(flags.ArgumentParser):
+  """Parser for --logger_levels flag."""
+
+  def parse(self, value):
+    if isinstance(value, abc.Mapping):
+      return value
+
+    pairs = [pair.strip() for pair in value.split(',') if pair.strip()]
+
+    # Preserve the order so that serialization is deterministic.
+    levels = collections.OrderedDict()
+    for name_level in pairs:
+      name, level = name_level.split(':', 1)
+      name = name.strip()
+      level = level.strip()
+      levels[name] = level
+    return levels
+
+
+class _LoggerLevelsSerializer(object):
+  """Serializer for --logger_levels flag."""
+
+  def serialize(self, value):
+    if isinstance(value, str):
+      return value
+    return ','.join(
+        '{}:{}'.format(name, level) for name, level in value.items())
+
+
 class _StderrthresholdFlag(flags.Flag):
   """Flag class for --stderrthreshold."""
 
@@ -210,7 +270,7 @@ class _StderrthresholdFlag(flags.Flag):
   @value.setter
   def value(self, v):
     if v in _CPP_LEVEL_TO_NAMES:
-      # --stderrthreshold also accepts numberic strings whose values are
+      # --stderrthreshold also accepts numeric strings whose values are
       # Abseil C++ log levels.
       cpp_value = int(v)
       v = _CPP_LEVEL_TO_NAMES[v]  # Normalize to strings.
@@ -245,6 +305,13 @@ flags.DEFINE_flag(_VerbosityFlag(
     'supplied, the value will be changed from the default of -1 (warning) to '
     '0 (info) after flags are parsed.',
     short_name='v', allow_hide_cpp=True))
+flags.DEFINE_flag(
+    _LoggerLevelsFlag(
+        'logger_levels', {},
+        'Specify log level of loggers. The format is a CSV list of '
+        '`name:level`. Where `name` is the logger name used with '
+        '`logging.getLogger()`, and `level` is a level name  (INFO, DEBUG, '
+        'etc). e.g. `myapp.foo:INFO,other.logger:DEBUG`'))
 flags.DEFINE_flag(_StderrthresholdFlag(
     'stderrthreshold', 'fatal',
     'log messages at this level, or more severe, to stderr in '
@@ -308,6 +375,7 @@ def set_stderrthreshold(s):
 
 
 def fatal(msg, *args, **kwargs):
+  # type: (Any, Any, Any) -> NoReturn
   """Logs a fatal message."""
   log(FATAL, msg, *args, **kwargs)
 
@@ -322,15 +390,11 @@ def warning(msg, *args, **kwargs):
   log(WARNING, msg, *args, **kwargs)
 
 
-if six.PY2:
-  warn = warning  # Deprecated function.
-else:
-
-  def warn(msg, *args, **kwargs):
-    """Deprecated, use 'warning' instead."""
-    warnings.warn("The 'warn' function is deprecated, use 'warning' instead",
-                  DeprecationWarning, 2)
-    log(WARNING, msg, *args, **kwargs)
+def warn(msg, *args, **kwargs):
+  """Deprecated, use 'warning' instead."""
+  warnings.warn("The 'warn' function is deprecated, use 'warning' instead",
+                DeprecationWarning, 2)
+  log(WARNING, msg, *args, **kwargs)
 
 
 def info(msg, *args, **kwargs):
@@ -592,6 +656,10 @@ def find_log_dir_and_names(program_name=None, log_dir=None):
 
   Returns:
     (log_dir, file_prefix, symlink_prefix)
+
+  Raises:
+    FileNotFoundError: raised in Python 3 when it cannot find a log directory.
+    OSError: raised in Python 2 when it cannot find a log directory.
   """
   if not program_name:
     # Strip the extension (foobar.par becomes foobar, and
@@ -628,6 +696,10 @@ def find_log_dir(log_dir=None):
         directory.  Otherwise if the --log_dir command-line flag is provided,
         the logfile will be created in that directory.  Otherwise the logfile
         will be created in a standard location.
+
+  Raises:
+    FileNotFoundError: raised in Python 3 when it cannot find a log directory.
+    OSError: raised in Python 2 when it cannot find a log directory.
   """
   # Get a list of possible log dirs (will try to use them in order).
   if log_dir:
@@ -644,7 +716,8 @@ def find_log_dir(log_dir=None):
   for d in dirs:
     if os.path.isdir(d) and os.access(d, os.W_OK):
       return d
-  _absl_logger.fatal("Can't find a writable directory for logs, tried %s", dirs)
+  raise FileNotFoundError(
+      "Can't find a writable directory for logs, tried %s" % dirs)
 
 
 def get_absl_log_prefix(record):
@@ -708,7 +781,7 @@ def skip_log_prefix(func):
     file_name = func_code.co_filename
     func_name = func_code.co_name
     func_lineno = func_code.co_firstlineno
-  elif isinstance(func, six.string_types):
+  elif isinstance(func, str):
     file_name = get_absl_logger().findCaller()[0]
     func_name = func
     func_lineno = None
@@ -752,10 +825,7 @@ class PythonHandler(logging.StreamHandler):
         os.getpid())
     filename = os.path.join(actual_log_dir, basename)
 
-    if six.PY2:
-      self.stream = open(filename, 'a')
-    else:
-      self.stream = open(filename, 'a', encoding='utf-8')
+    self.stream = open(filename, 'a', encoding='utf-8')
 
     # os.symlink is not available on Windows Python 2.
     if getattr(os, 'symlink', None):
@@ -942,7 +1012,7 @@ class ABSLLogger(logging.getLoggerClass()):
   """A logger that will create LogRecords while skipping some stack frames.
 
   This class maintains an internal list of filenames and method names
-  for use when determining who called the currently execuing stack
+  for use when determining who called the currently executing stack
   frame.  Any method names from specific source files are skipped when
   walking backwards through the stack.
 
@@ -985,16 +1055,13 @@ class ABSLLogger(logging.getLoggerClass()):
           (code.co_filename, code.co_name,
            code.co_firstlineno) not in f_to_skip and
           (code.co_filename, code.co_name) not in f_to_skip):
-        if six.PY2 and not stack_info:
-          return (code.co_filename, frame.f_lineno, code.co_name)
-        else:
-          sinfo = None
-          if stack_info:
-            out = io.StringIO()
-            out.write(u'Stack (most recent call last):\n')
-            traceback.print_stack(frame, file=out)
-            sinfo = out.getvalue().rstrip(u'\n')
-          return (code.co_filename, frame.f_lineno, code.co_name, sinfo)
+        sinfo = None
+        if stack_info:
+          out = io.StringIO()
+          out.write(u'Stack (most recent call last):\n')
+          traceback.print_stack(frame, file=out)
+          sinfo = out.getvalue().rstrip(u'\n')
+        return (code.co_filename, frame.f_lineno, code.co_name, sinfo)
       frame = frame.f_back
 
   def critical(self, msg, *args, **kwargs):
@@ -1011,9 +1078,8 @@ class ABSLLogger(logging.getLoggerClass()):
 
   def warn(self, msg, *args, **kwargs):
     """Logs 'msg % args' with severity 'WARN'."""
-    if six.PY3:
-      warnings.warn("The 'warn' method is deprecated, use 'warning' instead",
-                    DeprecationWarning, 2)
+    warnings.warn("The 'warn' method is deprecated, use 'warning' instead",
+                  DeprecationWarning, 2)
     self.log(logging.WARN, msg, *args, **kwargs)
 
   def warning(self, msg, *args, **kwargs):
@@ -1096,7 +1162,7 @@ def _get_thread_id():
   Returns:
     Thread ID unique to this process (unsigned)
   """
-  thread_id = _thread_lib.get_ident()
+  thread_id = threading.get_ident()
   return thread_id & _THREAD_ID_MASK
 
 
@@ -1146,6 +1212,7 @@ def use_absl_handler():
   if absl_handler not in logging.root.handlers:
     logging.root.addHandler(absl_handler)
     FLAGS['verbosity']._update_logging_levels()  # pylint: disable=protected-access
+    FLAGS['logger_levels']._update_logger_levels()  # pylint: disable=protected-access
 
 
 def _initialize():
