@@ -18,12 +18,10 @@ This module contains base classes and high-level functions for Abseil-style
 tests.
 """
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
+from collections import abc
 import contextlib
 import difflib
+import enum
 import errno
 import getpass
 import inspect
@@ -42,6 +40,8 @@ import sys
 import tempfile
 import textwrap
 import unittest
+from unittest import mock  # pylint: disable=unused-import Allow absltest.mock.
+from urllib import parse
 
 try:
   # The faulthandler module isn't always available, and pytype doesn't
@@ -56,14 +56,8 @@ except ImportError:
 from absl import app
 from absl import flags
 from absl import logging
-from absl._collections_abc import abc
-from absl._enum_module import enum
 from absl.testing import _pretty_print_reporter
 from absl.testing import xml_reporter
-from absl.third_party import unittest3_backport
-import six
-from six.moves import urllib
-from six.moves import xrange  # pylint: disable=redefined-builtin
 
 # Make typing an optional import to avoid it being a required dependency
 # in Python 2. Type checkers will still understand the imports.
@@ -81,21 +75,10 @@ else:
     # Unbounded TypeVar for general usage
     _T = typing.TypeVar('_T')
 
-    if six.PY2:
-      _OutcomeType = unittest3_backport.case._Outcome
-    else:
-      import unittest.case
-      _OutcomeType = unittest.case._Outcome  # pytype: disable=module-attr
+    import unittest.case
+    _OutcomeType = unittest.case._Outcome  # pytype: disable=module-attr
 
 
-if six.PY3:
-  from unittest import mock  # pylint: disable=unused-import
-else:
-  try:
-    import mock  # type: ignore
-  except ImportError:
-    mock = None
-
 
 # Re-export a bunch of unittest functions we support so that people don't
 # have to import unittest to get them
@@ -111,7 +94,7 @@ expectedFailure = unittest.expectedFailure
 
 FLAGS = flags.FLAGS
 
-_TEXT_OR_BINARY_TYPES = (six.text_type, six.binary_type)
+_TEXT_OR_BINARY_TYPES = (str, bytes)
 
 # Suppress surplus entries in AssertionError stack traces.
 __unittest = True  # pylint: disable=invalid-name
@@ -300,8 +283,8 @@ def _open(filepath, mode, _open_func=open):
   # type: (Text, Text, Callable[..., IO]) -> IO
   """Opens a file.
 
-  Like open(), but compatible with Python 2 and 3. Also ensures that we can open
-  real files even if tests stub out open().
+  Like open(), but ensure that we can open real files even if tests stub out
+  open().
 
   Args:
     filepath: A filepath.
@@ -311,10 +294,7 @@ def _open(filepath, mode, _open_func=open):
   Returns:
     The opened file object.
   """
-  if six.PY2:
-    return _open_func(filepath, mode)
-  else:
-    return _open_func(filepath, mode, encoding='utf-8')
+  return _open_func(filepath, mode, encoding='utf-8')
 
 
 class _TempDir(object):
@@ -394,7 +374,7 @@ class _TempDir(object):
 
     # Note: there's no need to clear the directory since the containing
     # dir was cleared by the tempdir() function.
-    _makedirs_exist_ok(path)
+    os.makedirs(path, exist_ok=True)
     return _TempDir(path)
 
 
@@ -422,14 +402,14 @@ class _TempFile(object):
     if file_path:
       cleanup_path = os.path.join(base_path, _get_first_part(file_path))
       path = os.path.join(base_path, file_path)
-      _makedirs_exist_ok(os.path.dirname(path))
+      os.makedirs(os.path.dirname(path), exist_ok=True)
       # The file may already exist, in which case, ensure it's writable so that
       # it can be truncated.
       if os.path.exists(path) and not os.access(path, os.W_OK):
         stat_info = os.stat(path)
         os.chmod(path, stat_info.st_mode | stat.S_IWUSR)
     else:
-      _makedirs_exist_ok(base_path)
+      os.makedirs(base_path, exist_ok=True)
       fd, path = tempfile.mkstemp(dir=str(base_path))
       os.close(fd)
       cleanup_path = path
@@ -437,7 +417,7 @@ class _TempFile(object):
     tf = cls(path)
 
     if content:
-      if isinstance(content, six.text_type):
+      if isinstance(content, str):
         tf.write_text(content, mode=mode, encoding=encoding, errors=errors)
       else:
         tf.write_bytes(content, mode)
@@ -486,8 +466,6 @@ class _TempFile(object):
       encoding: The encoding to use when writing the text to the file.
       errors: The error handling strategy to use when converting text to bytes.
     """
-    if six.PY2 and isinstance(text, bytes):
-      text = text.decode(encoding, errors)
     with self.open_text(mode, encoding=encoding, errors=errors) as fp:
       fp.write(text)
 
@@ -593,7 +571,7 @@ class _method(object):
     return func.__get__(obj, type_)  # pytype: disable=attribute-error
 
 
-class TestCase(unittest3_backport.TestCase):
+class TestCase(unittest.TestCase):
   """Extension of unittest.TestCase providing more power."""
 
   # When to cleanup files/directories created by our `create_tempfile()` and
@@ -677,12 +655,12 @@ class TestCase(unittest3_backport.TestCase):
       path = os.path.join(test_path, name)
       cleanup_path = os.path.join(test_path, _get_first_part(name))
     else:
-      _makedirs_exist_ok(test_path)
+      os.makedirs(test_path, exist_ok=True)
       path = tempfile.mkdtemp(dir=test_path)
       cleanup_path = path
 
     _rmtree_ignore_errors(cleanup_path)
-    _makedirs_exist_ok(path)
+    os.makedirs(path, exist_ok=True)
 
     self._maybe_add_temp_path_cleanup(cleanup_path, cleanup)
 
@@ -787,7 +765,8 @@ class TestCase(unittest3_backport.TestCase):
   @classmethod
   def _get_tempdir_path_cls(cls):
     # type: () -> Text
-    return os.path.join(TEST_TMPDIR.value, _get_qualname(cls))
+    return os.path.join(TEST_TMPDIR.value,
+                        cls.__qualname__.replace('__main__.', ''))
 
   def _get_tempdir_path_test(self):
     # type: () -> Text
@@ -1057,51 +1036,14 @@ class TestCase(unittest3_backport.TestCase):
   def assertItemsEqual(self, expected_seq, actual_seq, msg=None):
     """Deprecated, please use assertCountEqual instead.
 
-    This is equivalent to assertCountEqual in Python 3. An implementation of
-    assertCountEqual is also provided by absltest.TestCase for Python 2.
+    This is equivalent to assertCountEqual.
 
     Args:
       expected_seq: A sequence containing elements we are expecting.
       actual_seq: The sequence that we are testing.
       msg: The message to be printed if the test fails.
     """
-    if six.PY3:
-      # The assertItemsEqual method was renamed assertCountEqual in Python 3.2
-      super(TestCase, self).assertCountEqual(expected_seq, actual_seq, msg)
-    else:
-      super(TestCase, self).assertItemsEqual(expected_seq, actual_seq, msg)
-
-  # Only override assertCountEqual in Python 2 to avoid unnecessary calls.
-  if six.PY2:
-
-    def assertCountEqual(self, expected_seq, actual_seq, msg=None):
-      """Tests two sequences have the same elements regardless of order.
-
-      It tests that the first sequence contains the same elements as the
-      second, regardless of their order. When they don't, an error message
-      listing the differences between the sequences will be generated.
-
-      Duplicate elements are not ignored when comparing first and second.
-      It verifies whether each element has the same count in both sequences.
-      Equivalent to:
-
-          self.assertEqual(Counter(list(expected_seq)),
-                           Counter(list(actual_seq)))
-
-      but works with sequences of unhashable objects as well.
-
-      Example:
-          - [0, 1, 1] and [1, 0, 1] compare equal.
-          - [0, 0, 1] and [0, 1] compare unequal.
-
-      Args:
-        expected_seq: A sequence containing elements we are expecting.
-        actual_seq: The sequence that we are testing.
-        msg: The message to be printed if the test fails.
-
-      """
-      # Only call super's method to avoid potential infinite recursions.
-      super(TestCase, self).assertItemsEqual(expected_seq, actual_seq, msg)
+    super().assertCountEqual(expected_seq, actual_seq, msg)
 
   def assertSameElements(self, expected_seq, actual_seq, msg=None):
     """Asserts that two sequences have the same elements (in any order).
@@ -1161,10 +1103,10 @@ class TestCase(unittest3_backport.TestCase):
   # has a different error format. However, I find this slightly more readable.
   def assertMultiLineEqual(self, first, second, msg=None, **kwargs):
     """Asserts that two multi-line strings are equal."""
-    assert isinstance(first, six.string_types), (
-        'First argument is not a string: %r' % (first,))
-    assert isinstance(second, six.string_types), (
-        'Second argument is not a string: %r' % (second,))
+    assert isinstance(first,
+                      str), ('First argument is not a string: %r' % (first,))
+    assert isinstance(second,
+                      str), ('Second argument is not a string: %r' % (second,))
     line_limit = kwargs.pop('line_limit', 0)
     if kwargs:
       raise TypeError('Unexpected keyword args {}'.format(tuple(kwargs)))
@@ -1198,18 +1140,6 @@ class TestCase(unittest3_backport.TestCase):
     self.assertTrue(minv <= value, msg)
     self.assertTrue(maxv >= value, msg)
 
-  # Backport these names so that Py2 code can be written in Py3 style.
-  if six.PY2:
-
-    def assertRegex(self, *args, **kwargs):
-      return self.assertRegexpMatches(*args, **kwargs)
-
-    def assertRaisesRegex(self, *args, **kwargs):
-      return self.assertRaisesRegexp(*args, **kwargs)
-
-    def assertNotRegex(self, *args, **kwargs):
-      return self.assertNotRegexpMatches(*args, **kwargs)
-
   def assertRegexMatch(self, actual_str, regexes, message=None):
     r"""Asserts that at least one regex in regexes matches str.
 
@@ -1253,14 +1183,14 @@ class TestCase(unittest3_backport.TestCase):
       if type(regex) is not regex_type:  # pylint: disable=unidiomatic-typecheck
         self.fail('regexes list must all be the same type.', message)
 
-    if regex_type is bytes and isinstance(actual_str, six.text_type):
+    if regex_type is bytes and isinstance(actual_str, str):
       regexes = [regex.decode('utf-8') for regex in regexes]
-      regex_type = six.text_type
-    elif regex_type is six.text_type and isinstance(actual_str, bytes):
+      regex_type = str
+    elif regex_type is str and isinstance(actual_str, bytes):
       regexes = [regex.encode('utf-8') for regex in regexes]
       regex_type = bytes
 
-    if regex_type is six.text_type:
+    if regex_type is str:
       regex = u'(?:%s)' % u')|(?:'.join(regexes)
     elif regex_type is bytes:
       regex = b'(?:' + (b')|(?:'.join(regexes)) + b')'
@@ -1292,7 +1222,7 @@ class TestCase(unittest3_backport.TestCase):
     # We need bytes regexes here because `err` is bytes.
     # Accommodate code which listed their output regexes w/o the b'' prefix by
     # converting them to bytes for the user.
-    if isinstance(regexes[0], six.text_type):
+    if isinstance(regexes[0], str):
       regexes = [regex.encode('utf-8') for regex in regexes]
 
     command_string = get_command_string(command)
@@ -1338,7 +1268,7 @@ class TestCase(unittest3_backport.TestCase):
     # We need bytes regexes here because `err` is bytes.
     # Accommodate code which listed their output regexes w/o the b'' prefix by
     # converting them to bytes for the user.
-    if isinstance(regexes[0], six.text_type):
+    if isinstance(regexes[0], str):
       regexes = [regex.encode('utf-8') for regex in regexes]
 
     command_string = get_command_string(command)
@@ -1380,6 +1310,21 @@ class TestCase(unittest3_backport.TestCase):
       self.test_func(exc_value)
       return True
 
+  @typing.overload
+  def assertRaisesWithPredicateMatch(
+      self, expected_exception, predicate) -> _AssertRaisesContext:
+    # The purpose of this return statement is to work around
+    # https://github.com/PyCQA/pylint/issues/5273; it is otherwise ignored.
+    return self._AssertRaisesContext(None, None, None)
+
+  @typing.overload
+  def assertRaisesWithPredicateMatch(
+      self, expected_exception, predicate, callable_obj: Callable[..., Any],
+      *args, **kwargs) -> None:
+    # The purpose of this return statement is to work around
+    # https://github.com/PyCQA/pylint/issues/5273; it is otherwise ignored.
+    return self._AssertRaisesContext(None, None, None)
+
   def assertRaisesWithPredicateMatch(self, expected_exception, predicate,
                                      callable_obj=None, *args, **kwargs):
     """Asserts that exception is thrown and predicate(exception) is true.
@@ -1408,6 +1353,22 @@ class TestCase(unittest3_backport.TestCase):
     with context:
       callable_obj(*args, **kwargs)
 
+  @typing.overload
+  def assertRaisesWithLiteralMatch(
+      self, expected_exception, expected_exception_message
+  ) -> _AssertRaisesContext:
+    # The purpose of this return statement is to work around
+    # https://github.com/PyCQA/pylint/issues/5273; it is otherwise ignored.
+    return self._AssertRaisesContext(None, None, None)
+
+  @typing.overload
+  def assertRaisesWithLiteralMatch(
+      self, expected_exception, expected_exception_message,
+      callable_obj: Callable[..., Any], *args, **kwargs) -> None:
+    # The purpose of this return statement is to work around
+    # https://github.com/PyCQA/pylint/issues/5273; it is otherwise ignored.
+    return self._AssertRaisesContext(None, None, None)
+
   def assertRaisesWithLiteralMatch(self, expected_exception,
                                    expected_exception_message,
                                    callable_obj=None, *args, **kwargs):
@@ -1518,7 +1479,7 @@ class TestCase(unittest3_backport.TestCase):
     subsequence = list(subsequence)
     longest_match = 0
 
-    for start in xrange(1 + len(container) - len(subsequence)):
+    for start in range(1 + len(container) - len(subsequence)):
       if longest_match == len(subsequence):
         break
       index = 0
@@ -1625,7 +1586,7 @@ class TestCase(unittest3_backport.TestCase):
                                            '%r unexpectedly less than %r' %
                                            (b, a)))
       self.assertLessEqual(a, b, msg)
-      self.assertLessEqual(b, a, msg)
+      self.assertLessEqual(b, a, msg)  # pylint: disable=arguments-out-of-order
       self.assertFalse(a > b,
                        self._formatMessage(msg,
                                            '%r unexpectedly greater than %r' %
@@ -1635,7 +1596,7 @@ class TestCase(unittest3_backport.TestCase):
                                            '%r unexpectedly greater than %r' %
                                            (b, a)))
       self.assertGreaterEqual(a, b, msg)
-      self.assertGreaterEqual(b, a, msg)
+      self.assertGreaterEqual(b, a, msg)  # pylint: disable=arguments-out-of-order
 
     msg = kwargs.get('msg')
 
@@ -1682,8 +1643,8 @@ class TestCase(unittest3_backport.TestCase):
 
     if a == b:
       return
-    a_items = Sorted(list(six.iteritems(a)))
-    b_items = Sorted(list(six.iteritems(b)))
+    a_items = Sorted(list(a.items()))
+    b_items = Sorted(list(b.items()))
 
     unexpected = []
     missing = []
@@ -1695,8 +1656,7 @@ class TestCase(unittest3_backport.TestCase):
       """Deterministic repr for dict."""
       # Sort the entries based on their repr, not based on their sort order,
       # which will be non-deterministic across executions, for many types.
-      entries = sorted((safe_repr(k), safe_repr(v))
-                       for k, v in six.iteritems(dikt))
+      entries = sorted((safe_repr(k), safe_repr(v)) for k, v in dikt.items())
       return '{%s}' % (', '.join('%s: %s' % pair for pair in entries))
 
     message = ['%s != %s%s' % (Repr(a), Repr(b), ' (%s)' % msg if msg else '')]
@@ -1734,8 +1694,8 @@ class TestCase(unittest3_backport.TestCase):
 
   def assertUrlEqual(self, a, b, msg=None):
     """Asserts that urls are equal, ignoring ordering of query params."""
-    parsed_a = urllib.parse.urlparse(a)
-    parsed_b = urllib.parse.urlparse(b)
+    parsed_a = parse.urlparse(a)
+    parsed_b = parse.urlparse(b)
     self.assertEqual(parsed_a.scheme, parsed_b.scheme, msg)
     self.assertEqual(parsed_a.netloc, parsed_b.netloc, msg)
     self.assertEqual(parsed_a.path, parsed_b.path, msg)
@@ -1743,8 +1703,8 @@ class TestCase(unittest3_backport.TestCase):
     self.assertEqual(sorted(parsed_a.params.split(';')),
                      sorted(parsed_b.params.split(';')), msg)
     self.assertDictEqual(
-        urllib.parse.parse_qs(parsed_a.query, keep_blank_values=True),
-        urllib.parse.parse_qs(parsed_b.query, keep_blank_values=True), msg)
+        parse.parse_qs(parsed_a.query, keep_blank_values=True),
+        parse.parse_qs(parsed_b.query, keep_blank_values=True), msg)
 
   def assertSameStructure(self, a, b, aname='a', bname='b', msg=None):
     """Asserts that two values contain the same structural content.
@@ -1841,7 +1801,7 @@ def _sorted_list_difference(expected, actual):
 
   Args:
     expected:  The list we expected.
-    actual:  The list we actualy got.
+    actual:  The list we actually got.
   Returns:
     (missing, unexpected)
     missing: items in expected that are not in actual.
@@ -1882,7 +1842,7 @@ def _sorted_list_difference(expected, actual):
 
 def _are_both_of_integer_type(a, b):
   # type: (object, object) -> bool
-  return isinstance(a, six.integer_types) and isinstance(b, six.integer_types)
+  return isinstance(a, int) and isinstance(b, int)
 
 
 def _are_both_of_sequence_type(a, b):
@@ -1946,14 +1906,14 @@ def _walk_structure_for_problems(a, b, aname, bname, problem_list):
   elif (isinstance(a, abc.Sequence) and
         not isinstance(a, _TEXT_OR_BINARY_TYPES)):
     minlen = min(len(a), len(b))
-    for i in xrange(minlen):
+    for i in range(minlen):
       _walk_structure_for_problems(
           a[i], b[i], '%s[%d]' % (aname, i), '%s[%d]' % (bname, i),
           problem_list)
-    for i in xrange(minlen, len(a)):
+    for i in range(minlen, len(a)):
       problem_list.append('%s has [%i] with value %r but %s does not' %
                           (aname, i, a[i], bname))
-    for i in xrange(minlen, len(b)):
+    for i in range(minlen, len(b)):
       problem_list.append('%s lacks [%i] but %s has it with value %r' %
                           (aname, i, bname, b[i]))
 
@@ -1970,7 +1930,7 @@ def get_command_string(command):
   Returns:
     A string suitable for use as a shell command.
   """
-  if isinstance(command, six.string_types):
+  if isinstance(command, str):
     return command
   else:
     if os.name == 'nt':
@@ -2005,7 +1965,7 @@ def get_command_stderr(command, env=None, close_fds=True):
     # standard handles.
     close_fds = False
 
-  use_shell = isinstance(command, six.string_types)
+  use_shell = isinstance(command, str)
   process = subprocess.Popen(
       command,
       close_fds=close_fds,
@@ -2070,7 +2030,7 @@ def _is_in_app_main():
   """Returns True iff app.run is active."""
   f = sys._getframe().f_back  # pylint: disable=protected-access
   while f:
-    if f.f_code == six.get_function_code(app.run):  # pytype: disable=wrong-arg-types
+    if f.f_code == app.run.__code__:
       return True
     f = f.f_back
   return False
@@ -2093,7 +2053,7 @@ def _register_sigterm_with_faulthandler():
   # type: () -> None
   """Have faulthandler dump stacks on SIGTERM.  Useful to diagnose timeouts."""
   if faulthandler and getattr(faulthandler, 'register', None):
-    # faulthandler.register is not avaiable on Windows.
+    # faulthandler.register is not available on Windows.
     # faulthandler.enable() is already called by app.run.
     try:
       faulthandler.register(signal.SIGTERM, chain=True)  # pytype: disable=module-attr
@@ -2166,7 +2126,7 @@ def _run_in_app(function, args, kwargs):
     # after the command-line has been parsed. So we have the for loop below
     # to change back flags to their old values.
     argv = FLAGS(sys.argv)
-    for saved_flag in six.itervalues(saved_flags):
+    for saved_flag in saved_flags.values():
       saved_flag.restore_flag()
 
     function(argv, args, kwargs)
@@ -2188,15 +2148,10 @@ def _is_suspicious_attribute(testCaseClass, name):
   if name.startswith('Test') and len(name) > 4 and name[4].isupper():
     attr = getattr(testCaseClass, name)
     if inspect.isfunction(attr) or inspect.ismethod(attr):
-      if six.PY2:
-        args = inspect.getargspec(attr)
-        return (len(args.args) == 1 and args.args[0] == 'self'
-                and args.varargs is None and args.keywords is None)
-      else:
-        args = inspect.getfullargspec(attr)
-        return (len(args.args) == 1 and args.args[0] == 'self'
-                and args.varargs is None and args.varkw is None and
-                not args.kwonlyargs)
+      args = inspect.getfullargspec(attr)
+      return (len(args.args) == 1 and args.args[0] == 'self' and
+              args.varargs is None and args.varkw is None and
+              not args.kwonlyargs)
   return False
 
 
@@ -2336,8 +2291,12 @@ def _setup_filtering(argv):
   The following environment variable is used in this method:
 
     TESTBRIDGE_TEST_ONLY: string, if set, is forwarded to the unittest
-      framework to use as a test filter. Its value is split with shlex
-      before being passed as positional arguments on argv.
+      framework to use as a test filter. Its value is split with shlex, then:
+      1. On Python 3.6 and before, split values are passed as positional
+         arguments on argv.
+      2. On Python 3.7+, split values are passed to unittest's `-k` flag. Tests
+         are matched by glob patterns or substring. See
+         https://docs.python.org/3/library/unittest.html#cmdoption-unittest-k
 
   Args:
     argv: the argv to mutate in-place.
@@ -2346,7 +2305,11 @@ def _setup_filtering(argv):
   if argv is None or not test_filter:
     return
 
-  argv[1:1] = shlex.split(test_filter)
+  filters = shlex.split(test_filter)
+  if sys.version_info[:2] >= (3, 7):
+    filters = ['-k=' + test_filter for test_filter in filters]
+
+  argv[1:1] = filters
 
 
 def _setup_test_runner_fail_fast(argv):
@@ -2402,16 +2365,12 @@ def _setup_sharding(custom_loader=None):
   # to query whether a test binary implements the test sharding protocol.
   if 'TEST_SHARD_STATUS_FILE' in os.environ:
     try:
-      f = None
-      try:
-        f = open(os.environ['TEST_SHARD_STATUS_FILE'], 'w')
+      with open(os.environ['TEST_SHARD_STATUS_FILE'], 'w') as f:
         f.write('')
-      except IOError:
-        sys.stderr.write('Error opening TEST_SHARD_STATUS_FILE (%s). Exiting.'
-                         % os.environ['TEST_SHARD_STATUS_FILE'])
-        sys.exit(1)
-    finally:
-      if f is not None: f.close()
+    except IOError:
+      sys.stderr.write('Error opening TEST_SHARD_STATUS_FILE (%s). Exiting.'
+                       % os.environ['TEST_SHARD_STATUS_FILE'])
+      sys.exit(1)
 
   base_loader = custom_loader or TestLoader()
   if 'TEST_TOTAL_SHARDS' not in os.environ:
@@ -2430,7 +2389,7 @@ def _setup_sharding(custom_loader=None):
   # the test case names for this shard.
   delegate_get_names = base_loader.getTestCaseNames
 
-  bucket_iterator = itertools.cycle(xrange(total_shards))
+  bucket_iterator = itertools.cycle(range(total_shards))
 
   def getShardedTestCaseNames(testCaseClass):
     filtered_names = []
@@ -2503,7 +2462,7 @@ def _run_and_get_tests_result(argv, args, kwargs, xml_test_runner_class):
     # Use an in-memory buffer (not backed by the actual file) to store the XML
     # report, because some tools modify the file (e.g., create a placeholder
     # with partial information, in case the test process crashes).
-    xml_buffer = six.StringIO()
+    xml_buffer = io.StringIO()
     kwargs['testRunner'].set_default_xml_stream(xml_buffer)  # pytype: disable=attribute-error
 
     # If we've used a seed to randomize test case ordering, we want to record it
@@ -2578,15 +2537,6 @@ def run_tests(argv, args, kwargs):  # pylint: disable=line-too-long
   sys.exit(not result.wasSuccessful())
 
 
-def _get_qualname(cls):
-  # type: (Type) -> Text
-  if six.PY3:
-    name = cls.__qualname__
-  else:
-    name = '{}.{}'.format(cls.__module__, cls.__name__)
-  return name.replace('__main__.', '')
-
-
 def _rmtree_ignore_errors(path):
   # type: (Text) -> None
   if os.path.isfile(path):
@@ -2598,19 +2548,6 @@ def _rmtree_ignore_errors(path):
     shutil.rmtree(path, ignore_errors=True)
 
 
-def _makedirs_exist_ok(dir_name):
-  # type: (Text) -> None
-  if six.PY3:
-    os.makedirs(dir_name, exist_ok=True)  # pylint: disable=unexpected-keyword-arg
-  else:
-    # Python 2 doesn't have the exist_ok arg, so we roll it ourselves
-    try:
-      os.makedirs(dir_name)
-    except OSError as e:
-      if e.errno != errno.EEXIST:
-        raise
-
-
 def _get_first_part(path):
   # type: (Text) -> Text
   parts = path.split(os.sep, 1)
