@@ -33,7 +33,7 @@ like a normal test method, with its own setUp/tearDown cycle. In the
 example above, there are three separate testcases, one of which will
 fail due to an assertion error (1 + 1 != 3).
 
-Parameters for invididual test cases can be tuples (with positional parameters)
+Parameters for individual test cases can be tuples (with positional parameters)
 or dictionaries (with named parameters):
 
   class AdditionExample(parameterized.TestCase):
@@ -155,6 +155,37 @@ inside a tuple:
       self.assertEqual(0, sum(arg))
 
 
+Cartesian product of Parameter Values as Parametrized Test Cases
+======================================================
+If required to test method over a cartesian product of parameters,
+`parameterized.product` may be used to facilitate generation of parameters
+test combinations:
+
+  class TestModuloExample(parameterized.TestCase):
+    @parameterized.product(
+        num=[0, 20, 80],
+        modulo=[2, 4],
+        expected=[0]
+    )
+    def testModuloResult(self, num, modulo, expected):
+      self.assertEqual(expected, num % modulo)
+
+This results in 6 test cases being created - one for each combination of the
+parameters. It is also possible to supply sequences of keyword argument dicts
+as elements of the cartesian product:
+
+    @parameterized.product(
+        (dict(num=5, modulo=3, expected=2),
+         dict(num=7, modulo=4, expected=3)),
+        dtype=(int, float)
+    )
+    def testModuloResult(self, num, modulo, expected, dtype):
+      self.assertEqual(expected, dtype(num) % modulo)
+
+This results in 4 test cases being created - for each of the two sets of test
+data (supplied as kwarg dicts) and for each of the two data types (supplied as
+a named parameter). Multiple keyword argument dicts may be supplied if required.
+
 Async Support
 ===============================
 If a test needs to call async functions, it can inherit from both
@@ -173,23 +204,16 @@ as [asynctest](https://github.com/Martiusweb/asynctest):
       self.assertEqual(actual, expected)
 """
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
+from collections import abc
 import functools
+import inspect
+import itertools
 import re
 import types
 import unittest
 
-from absl._collections_abc import abc
 from absl.testing import absltest
-import six
 
-try:
-  from absl.testing import _parameterized_async
-except (ImportError, SyntaxError):
-  _parameterized_async = None
 
 _ADDR_RE = re.compile(r'\<([a-zA-Z0-9_\-\.]+) object at 0x[a-fA-F0-9]+\>')
 _NAMED = object()
@@ -218,21 +242,27 @@ def _clean_repr(obj):
 
 
 def _non_string_or_bytes_iterable(obj):
-  return (isinstance(obj, abc.Iterable) and
-          not isinstance(obj, six.text_type) and
-          not isinstance(obj, six.binary_type))
+  return (isinstance(obj, abc.Iterable) and not isinstance(obj, str) and
+          not isinstance(obj, bytes))
 
 
 def _format_parameter_list(testcase_params):
   if isinstance(testcase_params, abc.Mapping):
     return ', '.join('%s=%s' % (argname, _clean_repr(value))
-                     for argname, value in six.iteritems(testcase_params))
+                     for argname, value in testcase_params.items())
   elif _non_string_or_bytes_iterable(testcase_params):
     return ', '.join(map(_clean_repr, testcase_params))
   else:
     return _format_parameter_list((testcase_params,))
 
 
+def _async_wrapped(func):
+  @functools.wraps(func)
+  async def wrapper(*args, **kwargs):
+    return await func(*args, **kwargs)
+  return wrapper
+
+
 class _ParameterizedTestIter(object):
   """Callable and iterable class for producing new test cases."""
 
@@ -297,10 +327,11 @@ class _ParameterizedTestIter(object):
                 'Dict for named tests must contain key "%s"' % _NAMED_DICT_KEY)
           # Create a new dict to avoid modifying the supplied testcase_params.
           testcase_name = testcase_params[_NAMED_DICT_KEY]
-          testcase_params = {k: v for k, v in six.iteritems(testcase_params)
-                             if k != _NAMED_DICT_KEY}
+          testcase_params = {
+              k: v for k, v in testcase_params.items() if k != _NAMED_DICT_KEY
+          }
         elif _non_string_or_bytes_iterable(testcase_params):
-          if not isinstance(testcase_params[0], six.string_types):
+          if not isinstance(testcase_params[0], str):
             raise RuntimeError(
                 'The first element of named test parameters is the test name '
                 'suffix and must be a string')
@@ -337,9 +368,8 @@ class _ParameterizedTestIter(object):
           bound_param_test.__name__, _format_parameter_list(testcase_params))
       if test_method.__doc__:
         bound_param_test.__doc__ += '\n%s' % (test_method.__doc__,)
-      if (_parameterized_async and
-          _parameterized_async.iscoroutinefunction(test_method)):
-        return _parameterized_async.async_wrapped(bound_param_test)
+      if inspect.iscoroutinefunction(test_method):
+        return _async_wrapped(bound_param_test)
       return bound_param_test
 
     return (make_bound_param_test(c) for c in self.testcases)
@@ -353,7 +383,7 @@ def _modify_class(class_object, testcases, naming_type):
   # NOTE: _test_params_repr is private to parameterized.TestCase and it's
   # metaclass; do not use it outside of those classes.
   class_object._test_params_reprs = test_params_reprs = {}
-  for name, obj in six.iteritems(class_object.__dict__.copy()):
+  for name, obj in class_object.__dict__.copy().items():
     if (name.startswith(unittest.TestLoader.testMethodPrefix)
         and isinstance(obj, types.FunctionType)):
       delattr(class_object, name)
@@ -361,7 +391,7 @@ def _modify_class(class_object, testcases, naming_type):
       _update_class_dict_for_param_test_case(
           class_object.__name__, methods, test_params_reprs, name,
           _ParameterizedTestIter(obj, testcases, naming_type, name))
-      for meth_name, meth in six.iteritems(methods):
+      for meth_name, meth in methods.items():
         setattr(class_object, meth_name, meth)
 
 
@@ -447,14 +477,79 @@ def named_parameters(*testcases):
   return _parameter_decorator(_NAMED, testcases)
 
 
+def product(*kwargs_seqs, **testgrid):
+  """A decorator for running tests over cartesian product of parameters values.
+
+  See the module docstring for a usage example. The test will be run for every
+  possible combination of the parameters.
+
+  Args:
+    *kwargs_seqs: Each positional parameter is a sequence of keyword arg dicts;
+      every test case generated will include exactly one kwargs dict from each
+      positional parameter; these will then be merged to form an overall list
+      of arguments for the test case.
+    **testgrid: A mapping of parameter names and their possible values. Possible
+      values should given as either a list or a tuple.
+
+  Raises:
+    NoTestsError: Raised when the decorator generates no tests.
+
+  Returns:
+     A test generator to be handled by TestGeneratorMetaclass.
+  """
+
+  for name, values in testgrid.items():
+    assert isinstance(values, (list, tuple)), (
+        'Values of {} must be given as list or tuple, found {}'.format(
+            name, type(values)))
+
+  prior_arg_names = set()
+  for kwargs_seq in kwargs_seqs:
+    assert ((isinstance(kwargs_seq, (list, tuple))) and
+            all(isinstance(kwargs, dict) for kwargs in kwargs_seq)), (
+                'Positional parameters must be a sequence of keyword arg'
+                'dicts, found {}'
+                .format(kwargs_seq))
+    if kwargs_seq:
+      arg_names = set(kwargs_seq[0])
+      assert all(set(kwargs) == arg_names for kwargs in kwargs_seq), (
+          'Keyword argument dicts within a single parameter must all have the '
+          'same keys, found {}'.format(kwargs_seq))
+      assert not (arg_names & prior_arg_names), (
+          'Keyword argument dict sequences must all have distinct argument '
+          'names, found duplicate(s) {}'
+          .format(sorted(arg_names & prior_arg_names)))
+      prior_arg_names |= arg_names
+
+  assert not (prior_arg_names & set(testgrid)), (
+      'Arguments supplied in kwargs dicts in positional parameters must not '
+      'overlap with arguments supplied as named parameters; found duplicate '
+      'argument(s) {}'.format(sorted(prior_arg_names & set(testgrid))))
+
+  # Convert testgrid into a sequence of sequences of kwargs dicts and combine
+  # with the positional parameters.
+  # So foo=[1,2], bar=[3,4] --> [[{foo: 1}, {foo: 2}], [{bar: 3, bar: 4}]]
+  testgrid = (tuple({k: v} for v in vs) for k, vs in testgrid.items())
+  testgrid = tuple(kwargs_seqs) + tuple(testgrid)
+
+  # Create all possible combinations of parameters as a cartesian product
+  # of parameter values.
+  testcases = [
+      dict(itertools.chain.from_iterable(case.items()
+                                         for case in cases))
+      for cases in itertools.product(*testgrid)
+  ]
+  return _parameter_decorator(_ARGUMENT_REPR, testcases)
+
+
 class TestGeneratorMetaclass(type):
   """Metaclass for adding tests generated by parameterized decorators."""
 
-  def __new__(mcs, class_name, bases, dct):
+  def __new__(cls, class_name, bases, dct):
     # NOTE: _test_params_repr is private to parameterized.TestCase and it's
     # metaclass; do not use it outside of those classes.
     test_params_reprs = dct.setdefault('_test_params_reprs', {})
-    for name, obj in six.iteritems(dct.copy()):
+    for name, obj in dct.copy().items():
       if (name.startswith(unittest.TestLoader.testMethodPrefix) and
           _non_string_or_bytes_iterable(obj)):
         # NOTE: `obj` might not be a _ParameterizedTestIter in two cases:
@@ -495,7 +590,7 @@ class TestGeneratorMetaclass(type):
           # That's why it should only inherit it if it does not exist.
           test_params_reprs.setdefault(test_method, test_method_id)
 
-    return type.__new__(mcs, class_name, bases, dct)
+    return type.__new__(cls, class_name, bases, dct)
 
 
 def _update_class_dict_for_param_test_case(
@@ -537,8 +632,7 @@ def _update_class_dict_for_param_test_case(
     test_params_reprs[new_name] = getattr(func, '__x_params_repr__', '')
 
 
-@six.add_metaclass(TestGeneratorMetaclass)
-class TestCase(absltest.TestCase):
+class TestCase(absltest.TestCase, metaclass=TestGeneratorMetaclass):
   """Base class for test cases using the parameters decorator."""
 
   # visibility: private; do not call outside this class.
