@@ -27,8 +27,6 @@ import traceback
 import unittest
 from xml.sax import saxutils
 from absl.testing import _pretty_print_reporter
-from absl.third_party import unittest3_backport
-import six
 
 
 # See http://www.w3.org/TR/REC-xml/#NT-Char
@@ -75,7 +73,7 @@ def _escape_cdata(s):
   Returns:
     An escaped version of the input string.
   """
-  for char, escaped in six.iteritems(_control_character_conversions):
+  for char, escaped in _control_character_conversions.items():
     s = s.replace(char, escaped)
   return s.replace(']]>', ']] >')
 
@@ -91,13 +89,8 @@ def _iso8601_timestamp(timestamp):
   """
   if timestamp is None or timestamp < 0:
     return None
-  # Use utcfromtimestamp in PY2 because it doesn't have a built-in UTC object
-  if six.PY2:
-    return '%s+00:00' % datetime.datetime.utcfromtimestamp(
-        timestamp).isoformat()
-  else:
-    return datetime.datetime.fromtimestamp(
-        timestamp, tz=datetime.timezone.utc).isoformat()
+  return datetime.datetime.fromtimestamp(
+      timestamp, tz=datetime.timezone.utc).isoformat()
 
 
 def _print_xml_element_header(element, attributes, stream, indentation=''):
@@ -111,8 +104,8 @@ def _print_xml_element_header(element, attributes, stream, indentation=''):
   """
   stream.write('%s<%s' % (indentation, element))
   for attribute in attributes:
-    if len(attribute) == 2 \
-        and attribute[0] is not None and attribute[1] is not None:
+    if (len(attribute) == 2 and attribute[0] is not None and
+        attribute[1] is not None):
       stream.write(' %s="%s"' % (attribute[0], attribute[1]))
   stream.write('>\n')
 
@@ -166,8 +159,7 @@ class _TestCaseResult(object):
       full_class_name = match.group(2)
     else:
       class_name = unittest.util.strclass(test.__class__)
-      if ((six.PY3 and isinstance(test, unittest.case._SubTest)) or
-          (six.PY2 and isinstance(test, unittest3_backport.case._SubTest))):
+      if isinstance(test, unittest.case._SubTest):
         # If the test case is a _SubTest, the real TestCase instance is
         # available as _SubTest.test_case.
         class_name = unittest.util.strclass(test.test_case.__class__)
@@ -241,6 +233,7 @@ class _TestSuiteResult(object):
     self.error_counts = {}
     self.overall_start_time = -1
     self.overall_end_time = -1
+    self._testsuites_properties = {}
 
   def add_test_case_result(self, test_case_result):
     suite_name = type(test_case_result.test).__name__
@@ -248,10 +241,7 @@ class _TestSuiteResult(object):
       # _ErrorHolder is a special case created by unittest for class / module
       # level functions.
       suite_name = test_case_result.full_class_name.rsplit('.')[-1]
-    if ((six.PY3 and
-         isinstance(test_case_result.test, unittest.case._SubTest)) or
-        (six.PY2 and
-         isinstance(test_case_result.test, unittest3_backport.case._SubTest))):
+    if isinstance(test_case_result.test, unittest.case._SubTest):
       # If the test case is a _SubTest, the real TestCase instance is
       # available as _SubTest.test_case.
       suite_name = type(test_case_result.test.test_case).__name__
@@ -281,6 +271,12 @@ class _TestSuiteResult(object):
         ('timestamp', _iso8601_timestamp(self.overall_start_time)),
     ]
     _print_xml_element_header('testsuites', overall_attributes, stream)
+    if self._testsuites_properties:
+      stream.write('    <properties>\n')
+      for name, value in sorted(self._testsuites_properties.items()):
+        stream.write('      <property name="%s" value="%s"></property>\n' %
+                     (_escape_xml_attr(name), _escape_xml_attr(str(value))))
+      stream.write('    </properties>\n')
 
     for suite_name in self.suites:
       suite = self.suites[suite_name]
@@ -342,11 +338,13 @@ class _TextAndXMLTestResult(_pretty_print_reporter.TextTestResult):
   _TEST_CASE_RESULT_CLASS = _TestCaseResult
 
   def __init__(self, xml_stream, stream, descriptions, verbosity,
-               time_getter=_time_copy):
+               time_getter=_time_copy, testsuites_properties=None):
     super(_TextAndXMLTestResult, self).__init__(stream, descriptions, verbosity)
     self.xml_stream = xml_stream
     self.pending_test_case_results = {}
     self.suite = self._TEST_SUITE_RESULT_CLASS()
+    if testsuites_properties:
+      self.suite._testsuites_properties = testsuites_properties
     self.time_getter = time_getter
 
     # This lock guards any mutations on pending_test_case_results.
@@ -379,7 +377,7 @@ class _TextAndXMLTestResult(_pretty_print_reporter.TextTestResult):
   def stopTestRun(self):
     self.suite.set_end_time(self.time_getter())
     # All pending_test_case_results will be added to the suite and removed from
-    # the pending_test_case_results dictionary. Grabing the write lock to avoid
+    # the pending_test_case_results dictionary. Grabbing the write lock to avoid
     # results from being added during this process to avoid duplicating adds or
     # accidentally erasing newly appended pending results.
     with self._pending_test_case_results_lock:
@@ -461,12 +459,14 @@ class _TextAndXMLTestResult(_pretty_print_reporter.TextTestResult):
 
   def addError(self, test, err):
     super(_TextAndXMLTestResult, self).addError(test, err)
-    error_summary = ('error', err[0], err[1], self._exc_info_to_string(err))
+    error_summary = ('error', err[0], err[1],
+                     self._exc_info_to_string(err, test=test))
     self.add_pending_test_case_result(test, error_summary=error_summary)
 
   def addFailure(self, test, err):
     super(_TextAndXMLTestResult, self).addFailure(test, err)
-    error_summary = ('failure', err[0], err[1], self._exc_info_to_string(err))
+    error_summary = ('failure', err[0], err[1],
+                     self._exc_info_to_string(err, test=test))
     self.add_pending_test_case_result(test, error_summary=error_summary)
 
   def addSkip(self, test, reason):
@@ -476,7 +476,8 @@ class _TextAndXMLTestResult(_pretty_print_reporter.TextTestResult):
   def addExpectedFailure(self, test, err):
     super(_TextAndXMLTestResult, self).addExpectedFailure(test, err)
     if callable(getattr(test, 'recordProperty', None)):
-      test.recordProperty('EXPECTED_FAILURE', self._exc_info_to_string(err))
+      test.recordProperty('EXPECTED_FAILURE',
+                          self._exc_info_to_string(err, test=test))
     self.add_pending_test_case_result(test)
 
   def addUnexpectedSuccess(self, test):
@@ -492,10 +493,10 @@ class _TextAndXMLTestResult(_pretty_print_reporter.TextTestResult):
     if err is not None:
       if issubclass(err[0], test.failureException):
         error_summary = ('failure', err[0], err[1],
-                         self._exc_info_to_string(err))
+                         self._exc_info_to_string(err, test=test))
       else:
         error_summary = ('error', err[0], err[1],
-                         self._exc_info_to_string(err))
+                         self._exc_info_to_string(err, test=test))
     else:
       error_summary = None
     self.add_pending_test_case_result(subtest, error_summary=error_summary)
@@ -516,6 +517,7 @@ class TextAndXMLTestRunner(unittest.TextTestRunner):
   _TEST_RESULT_CLASS = _TextAndXMLTestResult
 
   _xml_stream = None
+  _testsuites_properties = {}
 
   def __init__(self, xml_stream=None, *args, **kwargs):
     """Initialize a TextAndXMLTestRunner.
@@ -551,5 +553,10 @@ class TextAndXMLTestRunner(unittest.TextTestRunner):
     if self._xml_stream is None:
       return super(TextAndXMLTestRunner, self)._makeResult()
     else:
-      return self._TEST_RESULT_CLASS(self._xml_stream, self.stream,
-                                     self.descriptions, self.verbosity)
+      return self._TEST_RESULT_CLASS(
+          self._xml_stream, self.stream, self.descriptions, self.verbosity,
+          testsuites_properties=self._testsuites_properties)
+
+  @classmethod
+  def set_testsuites_property(cls, key, value):
+    cls._testsuites_properties[key] = value
